/* 
 *	FrmSourceCode.c
 *
 *	Event handler and general functions for module FrmSourceCode
 *
 *	JSM - 11 jul 2004
 *
 */
#include <PalmOS.h>
#include "FrmSourceCode.h"
#include "lua5_rsc.h"


#include "palmstd_lib.h"
#include "PalmFld.h"

extern char * pScript; /* Pointer to the global LuaScript */


static void CopyScriptToField() {
	FldStrToField(FrmSourceFld, (char *) pScript);
}

static void CopyFieldToScript() {
	FieldPtr field;
	char *  ptext;
	UInt32 size;

	field = GetObjectPtr(FrmSourceFld);
	ptext = FldGetTextPtr(field);

	if (ptext != NULL) {
		if (pScript != NULL) {
			MemPtrFree(pScript);
			pScript = NULL;
		}
		size = StrLen(ptext) + 1;
		pScript = MemPtrNew(size);
	
		MemMove((void *)pScript, (void *) ptext, size-1);
		pScript[size] = '\0';
	}
	else
		pScript = NULL;
	
}


static void EditViewUpdateScrollBar ()
{
	UInt16 scrollPos;
	UInt16 textHeight;
	UInt16 fieldHeight;
	Int16 maxValue;
	FieldPtr fld;
	ScrollBarPtr bar;

	fld = GetObjectPtr (FrmSourceFld);
	bar = GetObjectPtr (FrmSourceSB);

	FldGetScrollValues (fld, &scrollPos, &textHeight,  &fieldHeight);

	if (textHeight > fieldHeight)
	{
		// On occasion, such as after deleting a multi-line selection of text,
		// the display might be the last few lines of a field followed by some
		// blank lines.  To keep the current position in place and allow the user
		// to "gracefully" scroll out of the blank area, the number of blank lines
		// visible needs to be added to max value.  Otherwise the scroll position
		// may be greater than maxValue, get pinned to maxvalue in SclSetScrollBar
		// resulting in the scroll bar and the display being out of sync.
		maxValue = (textHeight - fieldHeight) + FldGetNumberOfBlankLines (fld);
	}
	else if (scrollPos)
		maxValue = scrollPos;
	else
		maxValue = 0;

	SclSetScrollBar (bar, scrollPos, 0, maxValue, fieldHeight-1);
}



static void EditViewScroll (Int16 linesToScroll, Boolean updateScrollbar)
{
	UInt16				blankLines;
	FieldPtr			fld;

	fld = GetObjectPtr (FrmSourceFld);
	blankLines = FldGetNumberOfBlankLines (fld);

	if (linesToScroll < 0)
		FldScrollField (fld, -linesToScroll, winUp);
	else if (linesToScroll > 0)
		FldScrollField (fld, linesToScroll, winDown);

	// If there were blank lines visible at the end of the field
	// then we need to update the scroll bar.
	if (blankLines || updateScrollbar)
	{
		ErrNonFatalDisplayIf(blankLines && linesToScroll > 0, "blank lines when scrolling winDown");

		EditViewUpdateScrollBar();
	}
}


/*
 * FUNCTION: FrmSourceCodeHandleEvent
 *
 * DESCRIPTION:
 *
 * Event handler for the FrmSourceCode form.
 *
 * PARAMETERS:
 *
 * eventP
 *     a pointer to an EventType structure
 *
 * RETURNED:
 *     true if the event was handled and should not be passed to
 *     FrmHandleEvent
 */
extern Boolean FrmSourceCodeHandleEvent(EventType * eventP)
{
	Boolean handled = false;
	FormType * frmP;

	switch (eventP->eType)
	{
		case frmOpenEvent:
			frmP = FrmGetActiveForm();
			FrmDrawForm(frmP);
			CopyScriptToField();
			EditViewUpdateScrollBar();
			handled = true;
			break;
            
        case frmUpdateEvent:
			/* 
			 * To do any custom drawing here, first call
			 * FrmDrawForm(), then do your drawing, and
			 * then set handled to true. 
			 */
			break;
			
		case ctlSelectEvent:
		{
			switch(eventP->data.ctlSelect.controlID) {
				case FrmSrcCodeBtOk:
					CopyFieldToScript();
					FrmReturnToForm(0);
					handled = true;
					break;
			}
			break;
		}
		
		case sclRepeatEvent:
			EditViewScroll (eventP->data.sclRepeat.newValue -
							eventP->data.sclRepeat.value, false);
			break;
			
		case fldChangedEvent:
			EditViewUpdateScrollBar();
			break;
		
	}
    
	return handled;
	
}

